export class Main extends Indicator {

    constructor() {
        super();

        this.setProperty(Property.SeparateWindow, true);
        this.setProperty(Property.Minimum, 0);
        this.setProperty(Property.Maximum, 100);

        this.addInput("RSIPeriod", InputType.int, 14);
        this.addInput("PriceType", InputType.PriceType, PriceType.Bid);

        this.buffers = {
            RSIBuffer : this.addBuffer(),
        };

        this.arr = {
            PosBuffer : [],
            NegBuffer : [],
        };
    }

    onInit() {
        const  {RSIPeriod} = this.getInputs();

        this.buffers.RSIBuffer
            .setShape(Shape.Line)
            .setColor(Color.DodgerBlue)
            .setShortName("RSI(" + RSIPeriod + ")")
            .setLabel("RSI(" + RSIPeriod + ")")
            .setDrawBegin(RSIPeriod)
        ;
    }


    async onUpdate() {
        const  {RSIPeriod} = this.getInputs(),
            {RSIBuffer} = this.buffers,
            {PosBuffer, NegBuffer} = this.arr,
            {Close} = await Bar.load([Bar.Mode.Close], PriceType),
            barSize = Close.length;

        let i, countedBars = Bar.counted(), rel, negative, positive;

        if (barSize <= RSIPeriod) return 0;
        if (countedBars < 1)
            for (i = 1; i <= RSIPeriod; i++) RSIBuffer.set((barSize - i),0);
        i = barSize - RSIPeriod - 1;
        if (countedBars >= RSIPeriod) i = barSize - countedBars - 1;

        while (i >= 0) {
            let sumn = 0, sump = 0;
            if (i === barSize - RSIPeriod - 1) {
                let k = barSize - 2;
                while (k >= i) {
                    rel = Close[k] - Close[k + 1];
                    if (rel > 0) {
                        sump += rel;
                    }
                    else {
                        sumn -= rel;
                    }
                    k--;
                }
                positive = sump / RSIPeriod;
                negative = sumn / RSIPeriod;
            }
            else {
                rel = Close[i] - Close[i + 1];
                if (rel > 0){
                    sump = rel;
                }
                else  {
                    sumn = -rel;
                }
                positive = (PosBuffer[i + 1] * (RSIPeriod - 1) + sump) / RSIPeriod;
                negative = (NegBuffer[i + 1] * (RSIPeriod - 1) + sumn) / RSIPeriod;
            }
            PosBuffer[i] = positive;
            NegBuffer[i] = negative;
            if (negative === 0) {
                RSIBuffer.set(i, 0);
            }
            else {
                RSIBuffer.set(i, (100 - 100 / (1 + positive / negative)));
            }
            i--;
        }
    }
}